@@ -5,6 +5,7 @@ import time
 import argparse
 import os
 import sys
+import logging
 from .. import DirectoryChangeIterator, FileChangeIterator
 
 def get_parser():
@@ -15,16 +16,17 @@ def get_parser():
     parser.add_argument("-d", "--run-in-dir", dest="run_in_dir", default=None, help="Specifies a directory in which to run nose (defaults to the monitored directory)")
     parser.add_argument("-m", "--monitor-targets", dest="monitored_targets", default=[".",], help="Directories or files to monitor", action="append")
     parser.add_argument("-t", "--testing-utility", default="nosetests")
-    parser.add_argument("--shell", help="Treat the '-t' argument as a shell command", default=False, action="store_true")
     parser.add_argument("--ignore-pyc", default=False, action="store_true")
     parser.add_argument("-E", "--ext", "--extension", dest="accepted_extensions", action="append", default=[], help="Specifies extensions to track (like .py, etc.). Can be specified multiple times. NOTE: the default is to track .py and .pyc files. If this argument is given at least once, it replaces the default completely, so .py and .pyc should be passed again if desired.")
     parser.add_argument("remainder", nargs="*", default=None)
     return parser
 
 def main_loop(args):
+    logging.info("Gathering watch targets")
     watch_targets = _build_watch_targets(args)
     while True:
         for watch_target in watch_targets:
+            logging.info("Monitoring %s", watch_target)
             watch_target.run_if_changed()
             time.sleep(args.sleep)
     return 0
@@ -46,10 +48,14 @@ class WatchTarget(object):
         self.args = args
         self.target = target
         self._iterator = iter(_build_watcher(args, self.target, accepted_extensions))
+    def __repr__(self):
+        return repr(self.target)
     def run_if_changed(self):
         change = next(self._iterator)
         if not change:
+            logging.info("No change.")
             return
+        logging.info("Found change: %s elements", len(change))
         command = [self.args.testing_utility]
         command.extend(self.args.remainder)
         cwd = self.args.run_in_dir
@@ -59,8 +65,21 @@ class WatchTarget(object):
             cwd = os.path.dirname(cwd)
         if not cwd:
             cwd = '.'
-        p = subprocess.Popen(command, cwd=cwd, shell=self.args.shell)
-        p.wait()
+        logging.info("Running %r in %s", command, cwd)
+        p = subprocess.Popen(_get_shell_command_from_argv(command), cwd=cwd, shell=True)
+        result = p.wait()
+        logging.info("Result: %s", result)
+
+def _get_shell_command_from_argv(argv):
+    return " ".join(map(_quote_for_shell, argv))
+
+def _quote_for_shell(s):
+    s = s.replace("\\", "\\\\").replace('"', '\\"')
+    if " " in s:
+        s = '"{0}"'.format(s)
+    else:
+        s = s.replace("'", "\\'")
+    return s
 
 def _build_watcher(args, monitored_target, accepted_extensions):
     if os.path.isdir(monitored_target):
@@ -74,17 +93,11 @@ def _build_watcher(args, monitored_target, accepted_extensions):
 
 
 ################################## Boilerplate #################################
-_VERBOSE = False
-def log(msg, *args, **kwargs):
-    if _VERBOSE:
-        if args or kwargs:
-            msg = msg.format(*args, **kwargs)
-        print(msg, file=sys.stderr)
-
 
 def main():
-    global _VERBOSE
     args = get_parser().parse_args()
+    if args.verbose:
+        logging.basicConfig(level=logging.INFO, stream=sys.stderr)
     _VERBOSE = args.verbose
     try:
         sys.exit(main_loop(args))
